{% extends "layout.html" %}
{% set title = _('Index') %}
{% block body %}

   <h1 id="index">{% trans key=key %}Index &ndash; {{ key }}{% endtrans %}</h1>

<table width="100%" class="indextable"><tr><td width="33%" valign="top">
<dl>
{%- set breakat = count // 2 %}
{%- set numcols = 1 %}
{%- set numitems = 0 %}
{% for entryname, (links, subitems) in entries %}
<dt>{%- if links -%}<a href="{{ links[0] }}">{{ entryname|e }}</a>
  {%- for link in links[1:] %}, <a href="{{ link }}">[{{ loop.index }}]</a>{% endfor -%}
  {%- else -%}
{{ entryname|e }}
  {%- endif -%}</dt>
  {%- if subitems %}
  <dd><dl>
    {%- for subentryname, subentrylinks in subitems %}
    <dt><a href="{{ subentrylinks[0] }}">{{ subentryname|e }}</a>
    {%- for link in subentrylinks[1:] %}, <a href="{{ link }}">[{{ loop.index }}]</a>{% endfor -%}
    </dt>
    {%- endfor %}
  </dl></dd>
  {%- endif -%}
{%- set numitems = numitems + 1 + (subitems|length) -%}
{%- if numcols < 2 and numitems > breakat -%}
{%- set numcols = numcols+1 -%}
</dl></td><td width="33%" valign="top"><dl>
{%- endif -%}
{%- endfor %}
</dl></td></tr></table>

{% endblock %}

{% block sidebarrel %}
   <h4>Index</h4>
   <p>{% for key, dummy in genindexentries -%}
   <a href="{{ pathto('genindex-' + key) }}"><strong>{{ key }}</strong></a>
     {% if not loop.last %}| {% endif %}
   {%- endfor %}</p>

   <p><a href="{{ pathto('genindex-all') }}"><strong>{{ _('Full index on one page') }}</strong></a></p>
   {{ super() }}
{% endblock %}
